@@ -38,10 +38,10 @@ class VersionProps {
 
     // This field is read by HotSpot
     private static final String java_version =
-        "21.0.8";
+        "24.0.1";
 
     private static final String java_version_date =
-        "2025-07-15";
+        "2025-04-15";
 
     // This field is read by HotSpot
     private static final String java_runtime_name =
@@ -49,28 +49,28 @@ class VersionProps {
 
     // This field is read by HotSpot
     private static final String java_runtime_version =
-        "21.0.8+12-LTS-jvmci-23.1-b72";
+        "24.0.1+9-jvmci-b01";
 
     private static final String VERSION_NUMBER =
-        "21.0.8";
+        "24.0.1";
 
     private static final String VERSION_SPECIFICATION =
-        "21";
+        "24";
 
     private static final String VERSION_BUILD =
-        "12";
+        "9";
 
     private static final String VERSION_PRE =
         "";
 
     private static final String VERSION_OPT =
-        "LTS-jvmci-23.1-b72";
+        "jvmci-b01";
 
     private static final boolean isLTS =
-        "LTS-jvmci-23.1-b72".startsWith("LTS");
+        "jvmci-b01".startsWith("LTS");
 
     private static final String CLASSFILE_MAJOR_MINOR =
-        "65.0";
+        "68.0";
 
     private static final String VENDOR =
         "Oracle Corporation";
