@@ -3,9 +3,9 @@ package jdk.javadoc.internal.tool.resources;
 public final class version extends java.util.ListResourceBundle {
     protected final Object[][] getContents() {
         return new Object[][] {
-            { "full", "21.0.8+12-LTS-jvmci-23.1-b72" },
-            { "jdk", "21.0.8" },
-            { "release", "21.0.8" },
+            { "full", "24.0.1+9-jvmci-b01" },
+            { "jdk", "24.0.1" },
+            { "release", "24.0.1" },
         };
     }
 }
