@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007, 2020, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2024, Oracle and/or its affiliates. All rights reserved.
  * Copyright (c) 2020, 2020, Red Hat Inc. All rights reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
@@ -32,32 +32,24 @@ import java.io.File;
 import java.io.InputStream;
 import java.io.PrintStream;
 import java.net.URL;
-import java.util.ArrayList;
+import java.security.Permission;
 import java.util.Enumeration;
 import java.util.Map;
 import java.util.Properties;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentMap;
 import java.util.function.BooleanSupplier;
-import java.util.regex.Pattern;
 import java.util.stream.Stream;
 
-import org.graalvm.compiler.replacements.nodes.BinaryMathIntrinsicNode;
-import org.graalvm.compiler.replacements.nodes.BinaryMathIntrinsicNode.BinaryOperation;
-import org.graalvm.compiler.replacements.nodes.UnaryMathIntrinsicNode;
-import org.graalvm.compiler.replacements.nodes.UnaryMathIntrinsicNode.UnaryOperation;
 import org.graalvm.nativeimage.ImageSingletons;
 import org.graalvm.nativeimage.Platform;
 import org.graalvm.nativeimage.Platforms;
-import org.graalvm.nativeimage.c.function.CFunction;
-import org.graalvm.nativeimage.c.function.CLibrary;
 import org.graalvm.nativeimage.hosted.FieldValueTransformer;
 import org.graalvm.nativeimage.impl.InternalPlatform;
-import org.graalvm.word.WordBase;
-import org.graalvm.word.WordFactory;
 
-import com.oracle.svm.core.Containers;
+import com.oracle.svm.core.BuildPhaseProvider;
 import com.oracle.svm.core.NeverInline;
+import com.oracle.svm.core.NeverInlineTrivial;
 import com.oracle.svm.core.SubstrateOptions;
 import com.oracle.svm.core.SubstrateUtil;
 import com.oracle.svm.core.Uninterruptible;
@@ -70,9 +62,11 @@ import com.oracle.svm.core.annotate.RecomputeFieldValue.Kind;
 import com.oracle.svm.core.annotate.Substitute;
 import com.oracle.svm.core.annotate.TargetClass;
 import com.oracle.svm.core.annotate.TargetElement;
+import com.oracle.svm.core.container.Container;
+import com.oracle.svm.core.container.OperatingSystem;
+import com.oracle.svm.core.fieldvaluetransformer.FieldValueTransformerWithAvailability;
 import com.oracle.svm.core.hub.ClassForNameSupport;
 import com.oracle.svm.core.hub.DynamicHub;
-import com.oracle.svm.core.jdk.JavaLangSubstitutions.ClassValueSupport;
 import com.oracle.svm.core.monitor.MonitorSupport;
 import com.oracle.svm.core.snippets.SubstrateForeignCallTarget;
 import com.oracle.svm.core.thread.JavaThreads;
@@ -80,6 +74,10 @@ import com.oracle.svm.core.thread.VMOperation;
 import com.oracle.svm.core.util.VMError;
 import com.oracle.svm.util.ReflectionUtil;
 
+import jdk.graal.compiler.replacements.nodes.BinaryMathIntrinsicNode;
+import jdk.graal.compiler.replacements.nodes.BinaryMathIntrinsicNode.BinaryOperation;
+import jdk.graal.compiler.replacements.nodes.UnaryMathIntrinsicNode;
+import jdk.graal.compiler.replacements.nodes.UnaryMathIntrinsicNode.UnaryOperation;
 import jdk.internal.loader.ClassLoaderValue;
 
 @TargetClass(java.lang.Object.class)
@@ -88,7 +86,7 @@ final class Target_java_lang_Object {
 
     @Substitute
     @TargetElement(name = "getClass")
-    private Object getClassSubst() {
+    private DynamicHub getClassSubst() {
         return readHub(this);
     }
 
@@ -105,7 +103,6 @@ final class Target_java_lang_Object {
     }
 
     @Delete
-    @TargetElement(onlyWith = JDK19OrLater.class)
     private native void wait0(long timeoutMillis);
 
     @Substitute
@@ -136,11 +133,12 @@ final class Target_java_lang_Enum {
          * The original implementation creates and caches a HashMap to make the lookup faster. For
          * simplicity, we do a linear search for now.
          */
-        Enum<?>[] enumConstants = DynamicHub.fromClass(enumType).getEnumConstantsShared();
+        Object[] enumConstants = DynamicHub.fromClass(enumType).getEnumConstantsShared();
         if (enumConstants == null) {
             throw new IllegalArgumentException(enumType.getName() + " is not an enum type");
         }
-        for (Enum<?> e : enumConstants) {
+        for (Object o : enumConstants) {
+            Enum<?> e = (Enum<?>) o;
             if (e.name().equals(name)) {
                 return e;
             }
@@ -160,6 +158,16 @@ final class Target_java_lang_Enum {
 @TargetClass(java.lang.String.class)
 final class Target_java_lang_String {
 
+    // Checkstyle: stop
+    @Alias //
+    @RecomputeFieldValue(kind = RecomputeFieldValue.Kind.None, isFinal = true) //
+    public static boolean COMPACT_STRINGS;
+    // Checkstyle: resume
+
+    @Alias //
+    @RecomputeFieldValue(kind = RecomputeFieldValue.Kind.None, isFinal = true) //
+    public static byte LATIN1;
+
     @Substitute
     public String intern() {
         String thisStr = SubstrateUtil.cast(this, String.class);
@@ -183,78 +191,11 @@ final class Target_java_lang_String {
     native byte coder();
 
     @Alias @RecomputeFieldValue(kind = Kind.None, isFinal = true) //
-    byte[] value;
+    public byte[] value;
 
     @Alias //
     int hash;
 
-    /**
-     * This is a copy of String.split from the JDK, but with the fastpath loop factored out into a
-     * separate method. This allows inlining and constant folding of the condition for call sites
-     * where the regex is a constant (which is a common usage pattern).
-     *
-     * JDK-8262994 should make that refactoring in OpenJDK, after which this substitution can be
-     * removed.
-     */
-    @Substitute
-    public String[] split(String regex, int limit) {
-        /*
-         * fastpath if the regex is a (1) one-char String and this character is not one of the
-         * RegEx's meta characters ".$|()[{^?*+\\", or (2) two-char String and the first char is the
-         * backslash and the second is not the ascii digit or ascii letter.
-         */
-        char ch = 0;
-        if (((regex.length() == 1 &&
-                        ".$|()[{^?*+\\".indexOf(ch = regex.charAt(0)) == -1) ||
-                        (regex.length() == 2 &&
-                                        regex.charAt(0) == '\\' &&
-                                        (((ch = regex.charAt(1)) - '0') | ('9' - ch)) < 0 &&
-                                        ((ch - 'a') | ('z' - ch)) < 0 &&
-                                        ((ch - 'A') | ('Z' - ch)) < 0)) &&
-                        (ch < Character.MIN_HIGH_SURROGATE ||
-                                        ch > Character.MAX_LOW_SURROGATE)) {
-            return StringHelper.simpleSplit(SubstrateUtil.cast(this, String.class), limit, ch);
-        }
-        return Pattern.compile(regex).split(SubstrateUtil.cast(this, String.class), limit);
-    }
-}
-
-final class StringHelper {
-    static String[] simpleSplit(String that, int limit, char ch) {
-        int off = 0;
-        int next = 0;
-        boolean limited = limit > 0;
-        ArrayList<String> list = new ArrayList<>();
-        while ((next = that.indexOf(ch, off)) != -1) {
-            if (!limited || list.size() < limit - 1) {
-                list.add(that.substring(off, next));
-                off = next + 1;
-            } else {    // last one
-                // assert (list.size() == limit - 1);
-                int last = that.length();
-                list.add(that.substring(off, last));
-                off = last;
-                break;
-            }
-        }
-        // If no match was found, return this
-        if (off == 0) {
-            return new String[]{that};
-        }
-        // Add remaining segment
-        if (!limited || list.size() < limit) {
-            list.add(that.substring(off, that.length()));
-        }
-        // Construct result
-        int resultSize = list.size();
-        if (limit == 0) {
-            while (resultSize > 0 && list.get(resultSize - 1).isEmpty()) {
-                resultSize--;
-            }
-        }
-        String[] result = new String[resultSize];
-        return list.subList(0, resultSize).toArray(result);
-    }
 }
 
 @TargetClass(className = "java.lang.StringLatin1")
@@ -278,6 +219,11 @@ final class Target_java_lang_StringUTF16 {
 @SuppressWarnings({"unused"})
 final class Target_java_lang_Throwable {
 
+    @Alias //
+    @TargetElement(onlyWith = JDKLatest.class) //
+    @RecomputeFieldValue(kind = Kind.FromAlias, isFinal = true) //
+    static boolean jfrTracing = false;
+
     @Alias @RecomputeFieldValue(kind = Reset)//
     Object backtrace;
 
@@ -378,23 +324,9 @@ final class Target_java_lang_StackTraceElement {
      * @param depth ignored
      */
     @Substitute
-    @TargetElement(onlyWith = JDK19OrLater.class)
     static StackTraceElement[] of(Object x, int depth) {
         return StackTraceBuilder.build((long[]) x);
     }
-
-    /**
-     * Constructs the {@link StackTraceElement} array from a {@link Throwable}.
-     *
-     * @param t the {@link Throwable} object
-     * @param depth ignored
-     */
-    @Substitute
-    @TargetElement(onlyWith = JDK17OrEarlier.class)
-    static StackTraceElement[] of(Target_java_lang_Throwable t, int depth) {
-        Object x = t.backtrace;
-        return StackTraceBuilder.build((long[]) x);
-    }
 }
 
 @TargetClass(java.lang.Runtime.class)
@@ -413,11 +345,10 @@ final class Target_java_lang_Runtime {
             return optionValue;
         }
 
-        if (SubstrateOptions.MultiThreaded.getValue()) {
-            return Containers.activeProcessorCount();
-        } else {
-            return 1;
+        if (Container.singleton().isContainerized()) {
+            return Container.singleton().getActiveProcessorCount();
         }
+        return OperatingSystem.singleton().getActiveProcessorCount();
     }
 }
 
@@ -455,58 +386,73 @@ final class Target_java_lang_System {
 
     @Substitute
     private static Properties getProperties() {
-        return SystemPropertiesSupport.singleton().getCurrentProperties();
+        return SystemPropertiesSupport.singleton().getProperties();
     }
 
     @Substitute
     private static void setProperties(Properties props) {
-        SystemPropertiesSupport.singleton().setCurrentProperties(props);
+        SystemPropertiesSupport.singleton().setProperties(props);
     }
 
     @Substitute
     public static String setProperty(String key, String value) {
         checkKey(key);
-        return SystemPropertiesSupport.singleton().setCurrentProperty(key, value);
+        return SystemPropertiesSupport.singleton().setProperty(key, value);
     }
 
     @Substitute
+    @NeverInlineTrivial("Used in 'java.home' access analysis: AnalyzeJavaHomeAccessPhase")
     private static String getProperty(String key) {
         checkKey(key);
-        return SystemPropertiesSupport.singleton().getCurrentProperty(key);
+        return SystemPropertiesSupport.singleton().getProperty(key);
     }
 
     @Substitute
     public static String clearProperty(String key) {
         checkKey(key);
-        return SystemPropertiesSupport.singleton().clearCurrentProperty(key);
+        return SystemPropertiesSupport.singleton().clearProperty(key);
     }
 
     @Substitute
+    @NeverInlineTrivial("Used in 'java.home' access analysis: AnalyzeJavaHomeAccessPhase")
     private static String getProperty(String key, String def) {
-        checkKey(key);
-        return SystemPropertiesSupport.singleton().getCurrentProperty(key, def);
+        String result = getProperty(key);
+        return result != null ? result : def;
     }
 
     @Alias
     private static native void checkKey(String key);
 
-    /*
-     * Note that there is no substitution for getSecurityManager, but instead getSecurityManager it
-     * is intrinsified in SubstrateGraphBuilderPlugins to always return null. This allows better
-     * constant folding of SecurityManager code already during static analysis.
+    /**
+     * Force System.Never in case it was set at build time via the `-Djava.security.manager=allow`
+     * passed to the image builder.
+     */
+    @Alias @RecomputeFieldValue(kind = Kind.FromAlias, isFinal = true) //
+    @TargetElement(onlyWith = JDK21OrEarlier.class) private static int allowSecurityManager = 1;
+
+    /**
+     * We do not support the {@link SecurityManager} so this method must throw a
+     * {@link SecurityException} when 'java.security.manager' is set to anything but
+     * <code>disallow</code>.
+     * 
+     * @see System#setSecurityManager(SecurityManager)
+     * @see SecurityManager
      */
     @Substitute
-    private static void setSecurityManager(SecurityManager s) {
-        if (s != null) {
-            /*
-             * We deliberately treat this as a non-recoverable fatal error. We want to prevent bugs
-             * where an exception is silently ignored by an application and then necessary security
-             * checks are not in place.
-             */
-            throw VMError.shouldNotReachHere("Installing a SecurityManager is not yet supported");
+    @SuppressWarnings({"removal", "javadoc"})
+    @TargetElement(onlyWith = JDK21OrEarlier.class)
+    private static void setSecurityManager(SecurityManager sm) {
+        if (sm != null) {
+            /* Read the property collected at isolate creation as that is what happens on the JVM */
+            String smp = SystemPropertiesSupport.singleton().getSavedProperties().get("java.security.manager");
+            if (smp != null && !smp.equals("disallow")) {
+                throw new SecurityException("Setting the SecurityManager is not supported by Native Image");
+            } else {
+                throw new UnsupportedOperationException(
+                                "The Security Manager is deprecated and will be removed in a future release");
+            }
         }
     }
-
 }
 
 final class NotAArch64 implements BooleanSupplier {
@@ -516,6 +462,13 @@ final class NotAArch64 implements BooleanSupplier {
     }
 }
 
+final class IsAMD64 implements BooleanSupplier {
+    @Override
+    public boolean getAsBoolean() {
+        return Platform.includedIn(Platform.AMD64.class);
+    }
+}
+
 /**
  * When the intrinsics below are used outside of {@link java.lang.Math}, they are lowered to a
  * foreign call. This foreign call must be uninterruptible as it results from lowering a floating
@@ -544,6 +497,14 @@ final class Target_java_lang_Math {
         return UnaryMathIntrinsicNode.compute(a, UnaryOperation.TAN);
     }
 
+    @Substitute
+    @Uninterruptible(reason = "Must not contain a safepoint.")
+    @SubstrateForeignCallTarget(fullyUninterruptible = true, stubCallingConvention = false)
+    @TargetElement(onlyWith = IsAMD64.class)
+    public static double tanh(double a) {
+        return UnaryMathIntrinsicNode.compute(a, UnaryOperation.TANH);
+    }
+
     @Substitute
     @Uninterruptible(reason = "Must not contain a safepoint.")
     @SubstrateForeignCallTarget(fullyUninterruptible = true, stubCallingConvention = false)
@@ -573,159 +534,6 @@ final class Target_java_lang_Math {
     }
 }
 
-@TargetClass(value = StrictMath.class, onlyWith = JDK20OrEarlier.class)
-@Platforms(InternalPlatform.NATIVE_ONLY.class)
-final class Target_java_lang_StrictMath {
-
-    @Substitute
-    private static double sin(double a) {
-        return StrictMathInvoker.sin(WordFactory.nullPointer(), WordFactory.nullPointer(), a);
-    }
-
-    @Substitute
-    private static double cos(double a) {
-        return StrictMathInvoker.cos(WordFactory.nullPointer(), WordFactory.nullPointer(), a);
-    }
-
-    @Substitute
-    private static double tan(double a) {
-        return StrictMathInvoker.tan(WordFactory.nullPointer(), WordFactory.nullPointer(), a);
-    }
-
-    @Substitute
-    private static double asin(double a) {
-        return StrictMathInvoker.asin(WordFactory.nullPointer(), WordFactory.nullPointer(), a);
-    }
-
-    @Substitute
-    private static double acos(double a) {
-        return StrictMathInvoker.acos(WordFactory.nullPointer(), WordFactory.nullPointer(), a);
-    }
-
-    @Substitute
-    private static double atan(double a) {
-        return StrictMathInvoker.atan(WordFactory.nullPointer(), WordFactory.nullPointer(), a);
-    }
-
-    @Substitute
-    private static double log(double a) {
-        return StrictMathInvoker.log(WordFactory.nullPointer(), WordFactory.nullPointer(), a);
-    }
-
-    @Substitute
-    private static double log10(double a) {
-        return StrictMathInvoker.log10(WordFactory.nullPointer(), WordFactory.nullPointer(), a);
-    }
-
-    @Substitute
-    private static double sqrt(double a) {
-        return StrictMathInvoker.sqrt(WordFactory.nullPointer(), WordFactory.nullPointer(), a);
-    }
-
-    // Checkstyle: stop
-    @Substitute
-    private static double IEEEremainder(double f1, double f2) {
-        return StrictMathInvoker.IEEEremainder(WordFactory.nullPointer(), WordFactory.nullPointer(), f1, f2);
-    }
-    // Checkstyle: resume
-
-    @Substitute
-    private static double atan2(double y, double x) {
-        return StrictMathInvoker.atan2(WordFactory.nullPointer(), WordFactory.nullPointer(), y, x);
-    }
-
-    @Substitute
-    private static double sinh(double x) {
-        return StrictMathInvoker.sinh(WordFactory.nullPointer(), WordFactory.nullPointer(), x);
-    }
-
-    @Substitute
-    private static double cosh(double x) {
-        return StrictMathInvoker.cosh(WordFactory.nullPointer(), WordFactory.nullPointer(), x);
-    }
-
-    @Substitute
-    private static double tanh(double x) {
-        return StrictMathInvoker.tanh(WordFactory.nullPointer(), WordFactory.nullPointer(), x);
-    }
-
-    @Substitute
-    private static double expm1(double x) {
-        return StrictMathInvoker.expm1(WordFactory.nullPointer(), WordFactory.nullPointer(), x);
-    }
-
-    @Substitute
-    private static double log1p(double x) {
-        return StrictMathInvoker.log1p(WordFactory.nullPointer(), WordFactory.nullPointer(), x);
-    }
-}
-
-@CLibrary(value = "java", requireStatic = true, dependsOn = "fdlibm")
-final class StrictMathInvoker {
-
-    @CFunction(value = "Java_java_lang_StrictMath_sin", transition = CFunction.Transition.NO_TRANSITION)
-    static native double sin(WordBase jnienv, WordBase clazz, double a);
-
-    @CFunction(value = "Java_java_lang_StrictMath_cos", transition = CFunction.Transition.NO_TRANSITION)
-    static native double cos(WordBase jnienv, WordBase clazz, double a);
-
-    @CFunction(value = "Java_java_lang_StrictMath_tan", transition = CFunction.Transition.NO_TRANSITION)
-    static native double tan(WordBase jnienv, WordBase clazz, double a);
-
-    @CFunction(value = "Java_java_lang_StrictMath_asin", transition = CFunction.Transition.NO_TRANSITION)
-    static native double asin(WordBase jnienv, WordBase clazz, double a);
-
-    @CFunction(value = "Java_java_lang_StrictMath_acos", transition = CFunction.Transition.NO_TRANSITION)
-    static native double acos(WordBase jnienv, WordBase clazz, double a);
-
-    @CFunction(value = "Java_java_lang_StrictMath_atan", transition = CFunction.Transition.NO_TRANSITION)
-    static native double atan(WordBase jnienv, WordBase clazz, double a);
-
-    @CFunction(value = "Java_java_lang_StrictMath_exp", transition = CFunction.Transition.NO_TRANSITION)
-    static native double exp(WordBase jnienv, WordBase clazz, double a);
-
-    @CFunction(value = "Java_java_lang_StrictMath_log", transition = CFunction.Transition.NO_TRANSITION)
-    static native double log(WordBase jnienv, WordBase clazz, double a);
-
-    @CFunction(value = "Java_java_lang_StrictMath_log10", transition = CFunction.Transition.NO_TRANSITION)
-    static native double log10(WordBase jnienv, WordBase clazz, double a);
-
-    @CFunction(value = "Java_java_lang_StrictMath_sqrt", transition = CFunction.Transition.NO_TRANSITION)
-    static native double sqrt(WordBase jnienv, WordBase clazz, double a);
-
-    @CFunction(value = "Java_java_lang_StrictMath_cbrt", transition = CFunction.Transition.NO_TRANSITION)
-    static native double cbrt(WordBase jnienv, WordBase clazz, double a);
-
-    // Checkstyle: stop
-    @CFunction(value = "Java_java_lang_StrictMath_IEEEremainder", transition = CFunction.Transition.NO_TRANSITION)
-    static native double IEEEremainder(WordBase jnienv, WordBase clazz, double f1, double f2);
-    // Checkstyle: resume
-
-    @CFunction(value = "Java_java_lang_StrictMath_atan2", transition = CFunction.Transition.NO_TRANSITION)
-    static native double atan2(WordBase jnienv, WordBase clazz, double y, double x);
-
-    @CFunction(value = "Java_java_lang_StrictMath_pow", transition = CFunction.Transition.NO_TRANSITION)
-    static native double pow(WordBase jnienv, WordBase clazz, double a, double b);
-
-    @CFunction(value = "Java_java_lang_StrictMath_sinh", transition = CFunction.Transition.NO_TRANSITION)
-    static native double sinh(WordBase jnienv, WordBase clazz, double x);
-
-    @CFunction(value = "Java_java_lang_StrictMath_cosh", transition = CFunction.Transition.NO_TRANSITION)
-    static native double cosh(WordBase jnienv, WordBase clazz, double x);
-
-    @CFunction(value = "Java_java_lang_StrictMath_tanh", transition = CFunction.Transition.NO_TRANSITION)
-    static native double tanh(WordBase jnienv, WordBase clazz, double x);
-
-    @CFunction(value = "Java_java_lang_StrictMath_hypot", transition = CFunction.Transition.NO_TRANSITION)
-    static native double hypot(WordBase jnienv, WordBase clazz, double x, double y);
-
-    @CFunction(value = "Java_java_lang_StrictMath_expm1", transition = CFunction.Transition.NO_TRANSITION)
-    static native double expm1(WordBase jnienv, WordBase clazz, double x);
-
-    @CFunction(value = "Java_java_lang_StrictMath_log1p", transition = CFunction.Transition.NO_TRANSITION)
-    static native double log1p(WordBase jnienv, WordBase clazz, double x);
-}
-
 /**
  * We do not have dynamic class loading (and therefore no class unloading), so it is not necessary
  * to keep the complicated code that the JDK uses. However, our simple substitutions have a drawback
@@ -739,7 +547,7 @@ final class StrictMathInvoker {
 @Substitute
 final class Target_java_lang_ClassValue {
 
-    @RecomputeFieldValue(kind = RecomputeFieldValue.Kind.Custom, declClass = JavaLangSubstitutions.ClassValueInitializer.class)//
+    @RecomputeFieldValue(kind = RecomputeFieldValue.Kind.Custom, declClass = ClassValueInitializer.class)//
     private final ConcurrentMap<Class<?>, Object> values;
 
     @Substitute
@@ -779,32 +587,22 @@ final class Target_java_lang_ClassValue {
     }
 }
 
-@SuppressWarnings({"deprecation", "unused"})
-@TargetClass(className = "java.lang.Compiler", onlyWith = JDK20OrEarlier.class)
-final class Target_java_lang_Compiler {
-    @Substitute
-    static Object command(Object arg) {
-        return null;
+class ClassValueInitializer implements FieldValueTransformerWithAvailability {
+    @Override
+    public Object transform(Object receiver, Object originalValue) {
+        ClassValue<?> v = (ClassValue<?>) receiver;
+        Map<Class<?>, Object> map = ClassValueSupport.getValues().get(v);
+        assert map != null;
+        return map;
     }
 
-    @SuppressWarnings({"unused"})
-    @Substitute
-    static boolean compileClass(Class<?> clazz) {
-        return false;
-    }
-
-    @SuppressWarnings({"unused"})
-    @Substitute
-    static boolean compileClasses(String string) {
-        return false;
-    }
-
-    @Substitute
-    static void enable() {
-    }
-
-    @Substitute
-    static void disable() {
+    /*
+     * We want to wait to constant fold this value until all possible HotSpot initialization code
+     * has run.
+     */
+    @Override
+    public boolean isAvailable() {
+        return BuildPhaseProvider.isHostedUniverseBuilt();
     }
 }
 
@@ -840,6 +638,10 @@ final class Target_jdk_internal_loader_ClassLoaders {
 
 @TargetClass(value = jdk.internal.loader.BootLoader.class)
 final class Target_jdk_internal_loader_BootLoader {
+    // Checkstyle: stop
+    @Delete //
+    static String JAVA_HOME;
+    // Checkstyle: resume
 
     @Substitute
     static Package getDefinedPackage(String name) {
@@ -874,7 +676,7 @@ final class Target_jdk_internal_loader_BootLoader {
         return ClassForNameSupport.forNameOrNull(name, null);
     }
 
-    @SuppressWarnings("unused")
+    @SuppressWarnings({"unused", "restricted"})
     @Substitute
     private static void loadLibrary(String name) {
         System.loadLibrary(name);
@@ -905,6 +707,15 @@ final class Target_jdk_internal_loader_BootLoader {
     // Checkstyle: resume
 }
 
+@TargetClass(value = jdk.internal.logger.LoggerFinderLoader.class)
+final class Target_jdk_internal_logger_LoggerFinderLoader {
+    // Checkstyle: stop
+    @Alias @RecomputeFieldValue(kind = RecomputeFieldValue.Kind.Reset, isFinal = true)//
+    @TargetElement(onlyWith = JDK21OrEarlier.class)//
+    static Permission READ_PERMISSION;
+    // Checkstyle: resume
+}
+
 final class ClassLoaderValueMapFieldValueTransformer implements FieldValueTransformer {
     @Override
     public Object transform(Object receiver, Object originalValue) {
@@ -918,49 +729,4 @@ final class ClassLoaderValueMapFieldValueTransformer implements FieldValueTransf
 /** Dummy class to have a class with the file's name. */
 public final class JavaLangSubstitutions {
 
-    public static final class StringUtil {
-        /**
-         * Returns a character from a string at {@code index} position based on the encoding format.
-         */
-        @Uninterruptible(reason = "Called from uninterruptible code.", mayBeInlined = true)
-        public static char charAt(String string, int index) {
-            Target_java_lang_String str = SubstrateUtil.cast(string, Target_java_lang_String.class);
-            byte[] value = str.value;
-            if (str.isLatin1()) {
-                return Target_java_lang_StringLatin1.getChar(value, index);
-            } else {
-                return Target_java_lang_StringUTF16.getChar(value, index);
-            }
-        }
-
-        public static byte coder(String string) {
-            return SubstrateUtil.cast(string, Target_java_lang_String.class).coder();
-        }
-    }
-
-    public static final class ClassValueSupport {
-
-        /**
-         * Marker value that replaces null values in the
-         * {@link java.util.concurrent.ConcurrentHashMap}.
-         */
-        public static final Object NULL_MARKER = new Object();
-
-        final Map<ClassValue<?>, Map<Class<?>, Object>> values;
-
-        public ClassValueSupport(Map<ClassValue<?>, Map<Class<?>, Object>> map) {
-            values = map;
-        }
-    }
-
-    static class ClassValueInitializer implements FieldValueTransformer {
-        @Override
-        public Object transform(Object receiver, Object originalValue) {
-            ClassValueSupport support = ImageSingletons.lookup(ClassValueSupport.class);
-            ClassValue<?> v = (ClassValue<?>) receiver;
-            Map<Class<?>, Object> map = support.values.get(v);
-            assert map != null;
-            return map;
-        }
-    }
 }
